@@ -11,7 +11,7 @@
 #                                                                                                                      #
 # License:                                                                                                             #
 # ==================================================================================================================== #
-# Copyright 2017-2025 Patrick Lehmann - Bötzingen, Germany                                                             #
+# Copyright 2017-2026 Patrick Lehmann - Bötzingen, Germany                                                             #
 #                                                                                                                      #
 # Licensed under the Apache License, Version 2.0 (the "License");                                                      #
 # you may not use this file except in compliance with the License.                                                     #
@@ -31,7 +31,9 @@
 """
 Common platform information gathered from various sources.
 
-.. hint:: See :ref:`high-level help <COMMON/Platform>` for explanations and usage examples.
+.. hint::
+
+   See :ref:`high-level help <COMMON/Platform>` for explanations and usage examples.
 """
 from enum                    import Flag, auto, Enum
 
